@@ -30,12 +30,11 @@ from files.
 #include <stddef.h>
 #include <time.h>
 
-#include "arena.h"
-#include "config.h"
-#include "mem.h"
-#include "utils.h"
-#include "yara.h"
 
+#include <yara/arena.h>
+#include <yara/mem.h>
+#include <yara/error.h>
+#include <yara/limits.h>
 
 #define ARENA_FILE_VERSION      3
 
@@ -151,8 +150,8 @@ YR_ARENA_PAGE* _yr_arena_page_for_address(
 //
 // Args:
 //    YR_ARENA* arena    - Pointer the arena
-//    void* address   - Base address
-//    va_list offsets - List of offsets relative to base address
+//    void* address      - Base address
+//    va_list offsets    - List of offsets relative to base address
 //
 // Returns:
 //    ERROR_SUCCESS if succeed or the corresponding error code otherwise.
@@ -211,9 +210,9 @@ int _yr_arena_make_relocatable(
 //
 // Args:
 //    size_t initial_size  - Initial size
-//    int flags         - Flags
+//    int flags            - Flags
 //    YR_ARENA** arena     - Address where a pointer to the new arena will be
-//                        written to.
+//                           written to.
 //
 // Returns:
 //    ERROR_SUCCESS if succeed or the corresponding error code otherwise.
@@ -325,8 +324,8 @@ void* yr_arena_base_address(
 //
 // Args:
 //    YR_ARENA* arena  - Pointer to the arena.
-//    void* address - Base address.
-//    int offset    - Offset.
+//    void* address    - Base address.
+//    int offset       - Offset.
 //
 // Returns:
 //    A pointer
@@ -412,7 +411,7 @@ int yr_arena_coalesce(
 
   while(page != NULL)
   {
-    total_size += page->size;
+    total_size += page->used;
     page = page->next;
   }
 
@@ -489,30 +488,34 @@ int yr_arena_coalesce(
 
 
 //
-// yr_arena_allocate_memory
+// yr_arena_reserve_memory
 //
-// Allocates memory within the arena.
+// Ensures that the arena have enough contiguous memory for future allocations.
+// if the available space in the current page is lower than "size", a new page
+// is allocated.
 //
 // Args:
-//    YR_ARENA* arena - Pointer to the arena.
-//    size_t size - Size of the region to be allocated.
-//    void** allocated_memory - Address of a pointer to newly allocated
-//                              region.
+//    YR_ARENA* arena         - Pointer to the arena.
+//    size_t size             - Size of the region to be reserved.
+//
 // Returns:
 //    ERROR_SUCCESS if succeed or the corresponding error code otherwise.
 //
 
-int yr_arena_allocate_memory(
+
+int yr_arena_reserve_memory(
     YR_ARENA* arena,
-    size_t size,
-    void** allocated_memory)
+    size_t size)
 {
+  YR_ARENA_PAGE* new_page;
   size_t new_page_size;
   void* new_page_address;
-  YR_ARENA_PAGE* new_page;
 
   if (size > free_space(arena->current_page))
   {
+    if (arena->flags & ARENA_FLAGS_FIXED_SIZE)
+      return ERROR_INSUFICIENT_MEMORY;
+
     // Requested space is bigger than current page's empty space,
     // lets calculate the size for a new page.
 
@@ -537,9 +540,6 @@ int yr_arena_allocate_memory(
     }
     else
     {
-      if (arena->flags & ARENA_FLAGS_FIXED_SIZE)
-        return ERROR_INSUFICIENT_MEMORY;
-
       new_page = _yr_arena_new_page(new_page_size);
 
       if (new_page == NULL)
@@ -552,13 +552,37 @@ int yr_arena_allocate_memory(
     }
   }
 
+  return ERROR_SUCCESS;
+}
+
+
+//
+// yr_arena_allocate_memory
+//
+// Allocates memory within the arena.
+//
+// Args:
+//    YR_ARENA* arena         - Pointer to the arena.
+//    size_t size             - Size of the region to be allocated.
+//    void** allocated_memory - Address of a pointer to newly allocated
+//                              region.
+// Returns:
+//    ERROR_SUCCESS if succeed or the corresponding error code otherwise.
+//
+
+int yr_arena_allocate_memory(
+    YR_ARENA* arena,
+    size_t size,
+    void** allocated_memory)
+{
+  FAIL_ON_ERROR(yr_arena_reserve_memory(arena, size));
+
   *allocated_memory = arena->current_page->address + \
                       arena->current_page->used;
 
   arena->current_page->used += size;
 
   return ERROR_SUCCESS;
-
 }
 
 
@@ -582,12 +606,13 @@ int yr_arena_allocate_memory(
 //        EOL);
 //
 // Args:
-//    YR_ARENA* arena - Pointer to the arena.
-//    size_t size - Size of the region to be allocated.
+//    YR_ARENA* arena         - Pointer to the arena.
+//    size_t size             - Size of the region to be allocated.
 //    void** allocated_memory - Address of a pointer to newly allocated
 //                              region.
-//    ...          - Variable number of offsets relative to beginning of
-//                   the struct. Offsets are if type size_t.
+//    ...                     - Variable number of offsets relative to the
+//                              beginning of the struct. Offsets are of type
+//                              size_t.
 //
 // Returns:
 //    ERROR_SUCCESS if succeed or the corresponding error code otherwise.
@@ -624,9 +649,9 @@ int yr_arena_allocate_struct(
 //
 // Args:
 //    YR_ARENA* arena    - Pointer to the arena.
-//    void* base      - Address within the arena.
-//    ...             - Variable number of size_t arguments with offsets
-//                      relative to base.
+//    void* base         - Address within the arena.
+//    ...                - Variable number of size_t arguments with offsets
+//                         relative to base.
 //
 // Returns:
 //    ERROR_SUCCESS if succeed or the corresponding error code otherwise.
@@ -657,10 +682,10 @@ int yr_arena_make_relocatable(
 //
 // Args:
 //    YR_ARENA* arena        - Pointer to the arena.
-//    void* data          - Pointer to data to be written.
-//    size_t size        - Size of data.
-//    void** written_data - Address where a pointer to the written data will
-//                          be returned.
+//    void* data             - Pointer to data to be written.
+//    size_t size            - Size of data.
+//    void** written_data    - Address where a pointer to the written data will
+//                             be returned.
 //
 // Returns:
 //    ERROR_SUCCESS if succeed or the corresponding error code otherwise.
@@ -703,9 +728,9 @@ int yr_arena_write_data(
 // Writes string to the arena.
 //
 // Args:
-//    YR_ARENA* arena           - Pointer to the arena.
+//    YR_ARENA* arena        - Pointer to the arena.
 //    const char* string     - Pointer to string to be written.
-//    void** written_string  - Address where a pointer to the written data will
+//    char** written_string  - Address where a pointer to the written data will
 //                             be returned.
 //
 // Returns:
@@ -744,6 +769,7 @@ int yr_arena_append(
     YR_ARENA* source_arena)
 {
   target_arena->current_page->next = source_arena->page_list_head;
+  source_arena->page_list_head->prev = target_arena->current_page;
   target_arena->current_page = source_arena->current_page;
 
   yr_free(source_arena);
@@ -760,8 +786,8 @@ int yr_arena_append(
 //
 // Args:
 //    YR_ARENA* arena        - Pointer to the arena.
-//    YR_ARENA** duplicated  - Address where a pointer to the new arena arena will
-//                          be returned.
+//    YR_ARENA** duplicated  - Address where a pointer to the new arena arena
+//                             will be returned.
 //
 // Returns:
 //    ERROR_SUCCESS if succeed or the corresponding error code otherwise.
@@ -854,7 +880,7 @@ int yr_arena_duplicate(
 //
 // Args:
 //    YR_ARENA* arena          - Pointer to the arena.
-//    const char* filename  - File path.
+//    const char* filename     - File path.
 //
 // Returns:
 //    ERROR_SUCCESS if succeed or the corresponding error code otherwise.
@@ -946,7 +972,7 @@ int yr_arena_save(
 //
 // Args:
 //    const char* filename  - File path.
-//    YR_ARENA**               - Address where a pointer to the loaded arena
+//    YR_ARENA**            - Address where a pointer to the loaded arena
 //                            will be returned.
 //
 // Returns:
@@ -1057,8 +1083,3 @@ int yr_arena_load(
 
   return ERROR_SUCCESS;
 }
-
-
-
-
-
