@@ -16,7 +16,7 @@ limitations under the License.
 
 #include <fcntl.h>
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <windows.h>
 #else
 #include <sys/stat.h>
@@ -24,10 +24,11 @@ limitations under the License.
 #include <unistd.h>
 #endif
 
-#include "filemap.h"
-#include "yara.h"
+#include <yara/filemap.h>
+#include <yara/error.h>
 
-#ifdef WIN32
+
+#ifdef _WIN32
 
 //
 // Win32 implementation
@@ -35,8 +36,10 @@ limitations under the License.
 
 int yr_filemap_map(
     const char* file_path,
-    MAPPED_FILE* pmapped_file)
+    YR_MAPPED_FILE* pmapped_file)
 {
+  LARGE_INTEGER size;
+
   if (file_path == NULL)
     return ERROR_INVALID_ARGUMENT;
 
@@ -52,49 +55,68 @@ int yr_filemap_map(
   if (pmapped_file->file == INVALID_HANDLE_VALUE)
     return ERROR_COULD_NOT_OPEN_FILE;
 
-  pmapped_file->size = GetFileSize(pmapped_file->file, NULL);
-
-  if (pmapped_file->size == 0)
+  if (GetFileSizeEx(pmapped_file->file, &size))
+  {
+    #ifdef _WIN64
+    pmapped_file->size = size.QuadPart;
+    #else
+    pmapped_file->size = size.LowPart;
+    #endif
+  }
+  else
   {
     CloseHandle(pmapped_file->file);
-    return ERROR_ZERO_LENGTH_FILE;
+    return ERROR_COULD_NOT_OPEN_FILE;
   }
 
-  pmapped_file->mapping = CreateFileMapping(
-      pmapped_file->file,
-      NULL,
-      PAGE_READONLY,
-      0,
-      0,
-      NULL);
-
-  if (pmapped_file->mapping == NULL)
+  if (pmapped_file->size != 0)
   {
-    CloseHandle(pmapped_file->file);
-    return ERROR_COULD_NOT_MAP_FILE;
+    pmapped_file->mapping = CreateFileMapping(
+        pmapped_file->file,
+        NULL,
+        PAGE_READONLY,
+        0,
+        0,
+        NULL);
+
+    if (pmapped_file->mapping == NULL)
+    {
+      CloseHandle(pmapped_file->file);
+      return ERROR_COULD_NOT_MAP_FILE;
+    }
+
+    pmapped_file->data = (uint8_t*) MapViewOfFile(
+        pmapped_file->mapping,
+        FILE_MAP_READ,
+        0,
+        0,
+        0);
+
+    if (pmapped_file->data == NULL)
+    {
+      CloseHandle(pmapped_file->mapping);
+      CloseHandle(pmapped_file->file);
+      return ERROR_COULD_NOT_MAP_FILE;
+    }
   }
-
-  pmapped_file->data = (uint8_t*) MapViewOfFile(
-      pmapped_file->mapping,
-      FILE_MAP_READ,
-      0,
-      0,
-      0);
-
-  if (pmapped_file->data == NULL)
+  else
   {
-    CloseHandle(pmapped_file->mapping);
-    CloseHandle(pmapped_file->file);
-    return ERROR_COULD_NOT_MAP_FILE;
+    pmapped_file->mapping = NULL;
+    pmapped_file->data = NULL;
   }
 
   return ERROR_SUCCESS;
 }
 
-void yr_filemap_unmap(MAPPED_FILE* pmapped_file)
+void yr_filemap_unmap(
+    YR_MAPPED_FILE* pmapped_file)
 {
-  UnmapViewOfFile(pmapped_file->data);
-  CloseHandle(pmapped_file->mapping);
+  if (pmapped_file->data != NULL)
+    UnmapViewOfFile(pmapped_file->data);
+
+  if (pmapped_file->mapping != NULL)
+    CloseHandle(pmapped_file->mapping);
+
   CloseHandle(pmapped_file->file);
 }
 
@@ -106,7 +128,7 @@ void yr_filemap_unmap(MAPPED_FILE* pmapped_file)
 
 int yr_filemap_map(
     const char* file_path,
-    MAPPED_FILE* pmapped_file)
+    YR_MAPPED_FILE* pmapped_file)
 {
   struct stat fstat;
 
@@ -123,34 +145,37 @@ int yr_filemap_map(
 
   pmapped_file->size = fstat.st_size;
 
-  if (pmapped_file->size == 0)
+  if (pmapped_file->size != 0)
   {
-    close(pmapped_file->file);
-    return ERROR_ZERO_LENGTH_FILE;
+    pmapped_file->data = (uint8_t*) mmap(
+        0,
+        pmapped_file->size,
+        PROT_READ,
+        MAP_PRIVATE,
+        pmapped_file->file,
+        0);
+
+    if (pmapped_file->data == MAP_FAILED)
+    {
+      close(pmapped_file->file);
+      return ERROR_COULD_NOT_MAP_FILE;
+    }
   }
-
-  pmapped_file->data = (uint8_t*) mmap(
-      0,
-      pmapped_file->size,
-      PROT_READ,
-      MAP_PRIVATE,
-      pmapped_file->file,
-      0);
-
-  if (pmapped_file->data == MAP_FAILED)
+  else
   {
-    close(pmapped_file->file);
-    return ERROR_COULD_NOT_MAP_FILE;
+    pmapped_file->data = NULL;
   }
 
   return ERROR_SUCCESS;
 }
 
-void yr_filemap_unmap(MAPPED_FILE* pmapped_file)
+void yr_filemap_unmap(
+    YR_MAPPED_FILE* pmapped_file)
 {
-  munmap(pmapped_file->data, pmapped_file->size);
+  if (pmapped_file->data != NULL)
+    munmap(pmapped_file->data, pmapped_file->size);
+
   close(pmapped_file->file);
 }
 
 #endif
-
