@@ -45,61 +45,51 @@
      _META_ = 261,
      _STRINGS_ = 262,
      _CONDITION_ = 263,
-     _END_ = 264,
-     _IDENTIFIER_ = 265,
-     _STRING_IDENTIFIER_ = 266,
-     _STRING_COUNT_ = 267,
-     _STRING_OFFSET_ = 268,
-     _STRING_IDENTIFIER_WITH_WILDCARD_ = 269,
-     _ANONYMOUS_STRING_ = 270,
-     _NUMBER_ = 271,
-     _UNKNOWN_ = 272,
-     _TEXTSTRING_ = 273,
-     _HEXSTRING_ = 274,
-     _REGEXP_ = 275,
-     _ASCII_ = 276,
-     _WIDE_ = 277,
-     _NOCASE_ = 278,
-     _FULLWORD_ = 279,
-     _AT_ = 280,
-     _SIZE_ = 281,
-     _ENTRYPOINT_ = 282,
-     _ALL_ = 283,
-     _ANY_ = 284,
-     _RVA_ = 285,
-     _OFFSET_ = 286,
-     _FILE_ = 287,
-     _IN_ = 288,
-     _OF_ = 289,
-     _FOR_ = 290,
-     _THEM_ = 291,
-     _SECTION_ = 292,
-     _INT8_ = 293,
-     _INT16_ = 294,
-     _INT32_ = 295,
-     _UINT8_ = 296,
-     _UINT16_ = 297,
-     _UINT32_ = 298,
-     _MATCHES_ = 299,
-     _CONTAINS_ = 300,
-     _INDEX_ = 301,
-     _MZ_ = 302,
-     _PE_ = 303,
-     _DLL_ = 304,
-     _TRUE_ = 305,
-     _FALSE_ = 306,
-     _OR_ = 307,
-     _AND_ = 308,
-     _IS_ = 309,
-     _NEQ_ = 310,
-     _EQ_ = 311,
-     _GE_ = 312,
-     _GT_ = 313,
-     _LE_ = 314,
-     _LT_ = 315,
-     _SHIFT_RIGHT_ = 316,
-     _SHIFT_LEFT_ = 317,
-     _NOT_ = 318
+     _IDENTIFIER_ = 264,
+     _STRING_IDENTIFIER_ = 265,
+     _STRING_COUNT_ = 266,
+     _STRING_OFFSET_ = 267,
+     _STRING_IDENTIFIER_WITH_WILDCARD_ = 268,
+     _NUMBER_ = 269,
+     _TEXT_STRING_ = 270,
+     _HEX_STRING_ = 271,
+     _REGEXP_ = 272,
+     _ASCII_ = 273,
+     _WIDE_ = 274,
+     _NOCASE_ = 275,
+     _FULLWORD_ = 276,
+     _AT_ = 277,
+     _FILESIZE_ = 278,
+     _ENTRYPOINT_ = 279,
+     _ALL_ = 280,
+     _ANY_ = 281,
+     _IN_ = 282,
+     _OF_ = 283,
+     _FOR_ = 284,
+     _THEM_ = 285,
+     _INT8_ = 286,
+     _INT16_ = 287,
+     _INT32_ = 288,
+     _UINT8_ = 289,
+     _UINT16_ = 290,
+     _UINT32_ = 291,
+     _MATCHES_ = 292,
+     _CONTAINS_ = 293,
+     _IMPORT_ = 294,
+     _TRUE_ = 295,
+     _FALSE_ = 296,
+     _OR_ = 297,
+     _AND_ = 298,
+     _IS_ = 299,
+     _NEQ_ = 300,
+     _EQ_ = 301,
+     _GE_ = 302,
+     _GT_ = 303,
+     _LE_ = 304,
+     _LT_ = 305,
+     _SHIFT_RIGHT_ = 306,
+     _SHIFT_LEFT_ = 307,
+     _NOT_ = 308
    };
 #endif
 /* Tokens.  */
@@ -109,77 +99,69 @@
 #define _META_ 261
 #define _STRINGS_ 262
 #define _CONDITION_ 263
-#define _END_ 264
-#define _IDENTIFIER_ 265
-#define _STRING_IDENTIFIER_ 266
-#define _STRING_COUNT_ 267
-#define _STRING_OFFSET_ 268
-#define _STRING_IDENTIFIER_WITH_WILDCARD_ 269
-#define _ANONYMOUS_STRING_ 270
-#define _NUMBER_ 271
-#define _UNKNOWN_ 272
-#define _TEXTSTRING_ 273
-#define _HEXSTRING_ 274
-#define _REGEXP_ 275
-#define _ASCII_ 276
-#define _WIDE_ 277
-#define _NOCASE_ 278
-#define _FULLWORD_ 279
-#define _AT_ 280
-#define _SIZE_ 281
-#define _ENTRYPOINT_ 282
-#define _ALL_ 283
-#define _ANY_ 284
-#define _RVA_ 285
-#define _OFFSET_ 286
-#define _FILE_ 287
-#define _IN_ 288
-#define _OF_ 289
-#define _FOR_ 290
-#define _THEM_ 291
-#define _SECTION_ 292
-#define _INT8_ 293
-#define _INT16_ 294
-#define _INT32_ 295
-#define _UINT8_ 296
-#define _UINT16_ 297
-#define _UINT32_ 298
-#define _MATCHES_ 299
-#define _CONTAINS_ 300
-#define _INDEX_ 301
-#define _MZ_ 302
-#define _PE_ 303
-#define _DLL_ 304
-#define _TRUE_ 305
-#define _FALSE_ 306
-#define _OR_ 307
-#define _AND_ 308
-#define _IS_ 309
-#define _NEQ_ 310
-#define _EQ_ 311
-#define _GE_ 312
-#define _GT_ 313
-#define _LE_ 314
-#define _LT_ 315
-#define _SHIFT_RIGHT_ 316
-#define _SHIFT_LEFT_ 317
-#define _NOT_ 318
+#define _IDENTIFIER_ 264
+#define _STRING_IDENTIFIER_ 265
+#define _STRING_COUNT_ 266
+#define _STRING_OFFSET_ 267
+#define _STRING_IDENTIFIER_WITH_WILDCARD_ 268
+#define _NUMBER_ 269
+#define _TEXT_STRING_ 270
+#define _HEX_STRING_ 271
+#define _REGEXP_ 272
+#define _ASCII_ 273
+#define _WIDE_ 274
+#define _NOCASE_ 275
+#define _FULLWORD_ 276
+#define _AT_ 277
+#define _FILESIZE_ 278
+#define _ENTRYPOINT_ 279
+#define _ALL_ 280
+#define _ANY_ 281
+#define _IN_ 282
+#define _OF_ 283
+#define _FOR_ 284
+#define _THEM_ 285
+#define _INT8_ 286
+#define _INT16_ 287
+#define _INT32_ 288
+#define _UINT8_ 289
+#define _UINT16_ 290
+#define _UINT32_ 291
+#define _MATCHES_ 292
+#define _CONTAINS_ 293
+#define _IMPORT_ 294
+#define _TRUE_ 295
+#define _FALSE_ 296
+#define _OR_ 297
+#define _AND_ 298
+#define _IS_ 299
+#define _NEQ_ 300
+#define _EQ_ 301
+#define _GE_ 302
+#define _GT_ 303
+#define _LE_ 304
+#define _LT_ 305
+#define _SHIFT_RIGHT_ 306
+#define _SHIFT_LEFT_ 307
+#define _NOT_ 308
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 154 "grammar.y"
+#line 185 "grammar.y"
 {
   SIZED_STRING*   sized_string;
   char*           c_string;
+  int8_t          expression_type;
   int64_t         integer;
   YR_STRING*      string;
   YR_META*        meta;
+  YR_OBJECT*      object;
 }
 /* Line 1529 of yacc.c.  */
-#line 183 "grammar.h"
+#line 165 "grammar.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
