@@ -30,33 +30,39 @@ order to avoid confusion with operating system threads.
 #include <string.h>
 #include <limits.h>
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <windows.h>
 #else
 #include <pthread.h>
 #endif
 
-#include "yara.h"
-#include "arena.h"
-#include "mem.h"
-#include "re.h"
+#include <yara/limits.h>
+#include <yara/globals.h>
+#include <yara/utils.h>
+#include <yara/mem.h>
+#include <yara/re.h>
+#include <yara/error.h>
+#include <yara/re_lexer.h>
+#include <yara/hex_lexer.h>
 
 
-#define RE_MAX_STACK    1024
-#define RE_SCAN_LIMIT   4096
+#define RE_MAX_STACK      1024  // Maxium stack size for regexp evaluation
+#define RE_MAX_CODE_SIZE  16384 // Maximum code size for a compiled regexp
+#define RE_SCAN_LIMIT     4096  // Maximum input size scanned by yr_re_exec
 
-#define EMIT_FLAGS_BACKWARDS           1
-#define EMIT_FLAGS_DONT_ANNOTATE_RE    2
 
-#ifndef min
-#define min(x, y)  ((x < y) ? (x) : (y))
-#endif
+#define EMIT_BACKWARDS                  0x01
+#define EMIT_DONT_SET_FORWARDS_CODE     0x02
+#define EMIT_DONT_SET_BACKWARDS_CODE    0x04
+#define EMIT_NO_CASE                    0x08
+#define EMIT_DOT_ALL                    0x10
 
 
 typedef struct _RE_FIBER
 {
-  uint8_t* ip;
+  RE_CODE  ip;
   int32_t  sp;
+
   uint16_t stack[RE_MAX_STACK];
 
   struct _RE_FIBER* prev;
@@ -80,23 +86,12 @@ typedef struct _RE_THREAD_STORAGE
 } RE_THREAD_STORAGE;
 
 
-#ifdef WIN32
+#ifdef _WIN32
 DWORD thread_storage_key;
 #else
 pthread_key_t thread_storage_key;
 #endif
 
-
-extern int yr_parse_re_string(
-  const char* re_string,
-  RE** re);
-
-
-extern int yr_parse_hex_string(
-  const char* hex_string,
-  RE** re);
-
-
 //
 // yr_re_initialize
 //
@@ -104,9 +99,9 @@ extern int yr_parse_hex_string(
 // function from this module.
 //
 
-int yr_re_initialize()
+int yr_re_initialize(void)
 {
-  #ifdef WIN32
+  #ifdef _WIN32
   thread_storage_key = TlsAlloc();
   #else
   pthread_key_create(&thread_storage_key, NULL);
@@ -122,9 +117,9 @@ int yr_re_initialize()
 // stopped using functions from this module.
 //
 
-int yr_re_finalize()
+int yr_re_finalize(void)
 {
-  #ifdef WIN32
+  #ifdef _WIN32
   TlsFree(thread_storage_key);
   #else
   pthread_key_delete(thread_storage_key);
@@ -140,13 +135,13 @@ int yr_re_finalize()
 // before exiting.
 //
 
-int yr_re_finalize_thread()
+int yr_re_finalize_thread(void)
 {
   RE_FIBER* fiber;
   RE_FIBER* next_fiber;
   RE_THREAD_STORAGE* storage;
 
-  #ifdef WIN32
+  #ifdef _WIN32
   storage = TlsGetValue(thread_storage_key);
   #else
   storage = pthread_getspecific(thread_storage_key);
@@ -166,7 +161,7 @@ int yr_re_finalize_thread()
     yr_free(storage);
   }
 
-  #ifdef WIN32
+  #ifdef _WIN32
   TlsSetValue(thread_storage_key, NULL);
   #else
   pthread_setspecific(thread_storage_key, NULL);
@@ -198,7 +193,7 @@ RE_NODE* yr_re_node_create(
 
 
 void yr_re_node_destroy(
-  RE_NODE* node)
+    RE_NODE* node)
 {
   if (node->left != NULL)
     yr_re_node_destroy(node->left);
@@ -223,41 +218,137 @@ int yr_re_create(
 
   (*re)->flags = 0;
   (*re)->root_node = NULL;
-  (*re)->error_message = NULL;
-  (*re)->error_code = ERROR_SUCCESS;
+  (*re)->code_arena = NULL;
+  (*re)->code = NULL;
 
   return ERROR_SUCCESS;
 }
 
 
 void yr_re_destroy(
-  RE* re)
+    RE* re)
 {
   if (re->root_node != NULL)
     yr_re_node_destroy(re->root_node);
 
-  if (re->error_message != NULL)
-    yr_free((char*) re->error_message);
+  if (re->code_arena != NULL)
+    yr_arena_destroy(re->code_arena);
 
   yr_free(re);
 }
 
 
+//
+// yr_re_parse
+//
+// Parses a regexp but don't emit its code. A further call to y
+// yr_re_emit_code is required to get the code.
+//
+
+int yr_re_parse(
+    const char* re_string,
+    int flags,
+    RE** re,
+    RE_ERROR* error)
+{
+  return yr_parse_re_string(re_string, flags, re, error);
+}
+
+
+//
+// yr_re_parse_hex
+//
+// Parses a hex string but don't emit its code. A further call to
+// yr_re_emit_code is required to get the code.
+//
+
+int yr_re_parse_hex(
+    const char* hex_string,
+    int flags,
+    RE** re,
+    RE_ERROR* error)
+{
+  return yr_parse_hex_string(hex_string, flags, re, error);
+}
+
+
+//
+// yr_re_compile
+//
+// Parses the regexp and emit its code to the provided code_arena,
+// if code_arena is NULL the function creates a new arena of its own.
+//
+
 int yr_re_compile(
     const char* re_string,
-    RE** re)
+    int flags,
+    YR_ARENA* code_arena,
+    RE** re,
+    RE_ERROR* error)
 {
-  return yr_parse_re_string(re_string, re);
+  RE* compiled_re;
+  YR_ARENA* arena;
+
+  *re = NULL;
+
+  FAIL_ON_ERROR(yr_re_parse(re_string, flags, &compiled_re, error));
+
+  if (code_arena == NULL)
+  {
+    FAIL_ON_ERROR_WITH_CLEANUP(
+        yr_arena_create(
+            RE_MAX_CODE_SIZE,
+            ARENA_FLAGS_FIXED_SIZE,
+            &arena),
+        yr_re_destroy(compiled_re));
+
+    compiled_re->code_arena = arena;
+  }
+  else
+  {
+    arena = code_arena;
+  }
+
+  FAIL_ON_ERROR_WITH_CLEANUP(
+      yr_re_emit_code(compiled_re, arena),
+      yr_re_destroy(compiled_re));
+
+  *re = compiled_re;
+
+  return ERROR_SUCCESS;
 }
 
 
-int yr_re_compile_hex(
-    const char* hex_string,
-    RE** re)
+//
+// yr_re_match
+//
+// Verifies if the target string matches the pattern
+//
+// Args:
+//    uint8_t* re_code    -  A pointer to regexp code
+//    char* target        -  Target string
+//
+// Returns:
+//    Integer indicating the number of matching bytes, including 0 when
+//    matching an empty regexp. Negative values indicate:
+//      -1  No match
+//      -2  An error ocurred
+
+
+int yr_re_match(
+    RE_CODE re_code,
+    const char* target)
 {
-  return yr_parse_hex_string(hex_string, re);
+  return yr_re_exec(
+      re_code,
+      (uint8_t*) target,
+      strlen(target),
+      RE_FLAGS_SCAN,
+      NULL,
+      NULL);
 }
 
+
 //
 // yr_re_extract_literal
 //
@@ -358,6 +449,7 @@ int yr_re_split_at_chaining_point(
         return result;
 
       (*remainder_re)->root_node = child->left;
+      (*remainder_re)->flags = re->flags;
 
       child->left = NULL;
 
@@ -535,7 +627,9 @@ int _yr_re_emit(
 
     FAIL_ON_ERROR(_yr_emit_inst_arg_uint8(
         arena,
-        RE_OPCODE_LITERAL,
+        flags & EMIT_NO_CASE ?
+          RE_OPCODE_LITERAL_NO_CASE :
+          RE_OPCODE_LITERAL,
         re_node->value,
         &instruction_addr,
         NULL,
@@ -611,7 +705,9 @@ int _yr_re_emit(
 
     FAIL_ON_ERROR(_yr_emit_inst(
         arena,
-        RE_OPCODE_ANY,
+        flags & EMIT_DOT_ALL ?
+          RE_OPCODE_ANY :
+          RE_OPCODE_ANY_EXCEPT_NEW_LINE,
         &instruction_addr,
         code_size));
     break;
@@ -620,7 +716,9 @@ int _yr_re_emit(
 
     FAIL_ON_ERROR(_yr_emit_inst(
         arena,
-        RE_OPCODE_CLASS,
+        (flags & EMIT_NO_CASE) ?
+          RE_OPCODE_CLASS_NO_CASE :
+          RE_OPCODE_CLASS,
         &instruction_addr,
         code_size));
 
@@ -653,7 +751,7 @@ int _yr_re_emit(
 
   case RE_NODE_CONCAT:
 
-    if (flags & EMIT_FLAGS_BACKWARDS)
+    if (flags & EMIT_BACKWARDS)
     {
       left = re_node->right;
       right = re_node->left;
@@ -842,10 +940,15 @@ int _yr_re_emit(
 
       for (i = 0; i < re_node->start - 1; i++)
       {
+        // Don't want re_node->forward_code updated in this call
+        // forward_code must remain pointing to the code generated by
+        // by the  _yr_re_emit above. However we want re_node->backward_code
+        // being updated.
+
         FAIL_ON_ERROR(_yr_re_emit(
             re_node->left,
             arena,
-            flags | EMIT_FLAGS_DONT_ANNOTATE_RE,
+            flags | EMIT_DONT_SET_FORWARDS_CODE,
             NULL,
             &branch_size));
 
@@ -880,7 +983,7 @@ int _yr_re_emit(
     FAIL_ON_ERROR(_yr_re_emit(
         re_node->left,
         arena,
-        flags | EMIT_FLAGS_DONT_ANNOTATE_RE,
+        flags | EMIT_DONT_SET_FORWARDS_CODE | EMIT_DONT_SET_BACKWARDS_CODE,
         NULL,
         &branch_size));
 
@@ -907,11 +1010,14 @@ int _yr_re_emit(
     break;
   }
 
-  if (!(flags & EMIT_FLAGS_DONT_ANNOTATE_RE))
+  if (flags & EMIT_BACKWARDS)
   {
-    if (flags & EMIT_FLAGS_BACKWARDS)
-      re_node->backward_code = instruction_addr;
-    else
+    if (!(flags & EMIT_DONT_SET_BACKWARDS_CODE))
+      re_node->backward_code = instruction_addr + *code_size;
+  }
+  else
+  {
+    if (!(flags & EMIT_DONT_SET_FORWARDS_CODE))
       re_node->forward_code = instruction_addr;
   }
 
@@ -927,35 +1033,70 @@ int yr_re_emit_code(
     YR_ARENA* arena)
 {
   int code_size;
+  int total_size;
+
+  int emit_flags = 0;
+
+  if (re->flags & RE_FLAGS_NO_CASE)
+    emit_flags |= EMIT_NO_CASE;
+
+  if (re->flags & RE_FLAGS_DOT_ALL)
+    emit_flags |= EMIT_DOT_ALL;
+
+  // Ensure that we have enough contiguos memory space in the arena to
+  // contain the regular expression code. The code can't span over multiple
+  // non-contiguos pages.
+
+  yr_arena_reserve_memory(arena, RE_MAX_CODE_SIZE);
 
   // Emit code for matching the regular expressions forwards.
+
+  total_size = 0;
+
   FAIL_ON_ERROR(_yr_re_emit(
       re->root_node,
       arena,
-      0,
-      NULL,
+      emit_flags,
+      &re->code,
       &code_size));
 
+  total_size += code_size;
+
   FAIL_ON_ERROR(_yr_emit_inst(
       arena,
       RE_OPCODE_MATCH,
       NULL,
       &code_size));
 
+  total_size += code_size;
+
+  assert(total_size < RE_MAX_CODE_SIZE);
+
+  yr_arena_reserve_memory(arena, RE_MAX_CODE_SIZE);
+
   // Emit code for matching the regular expressions backwards.
+
+  total_size = 0;
+
   FAIL_ON_ERROR(_yr_re_emit(
       re->root_node,
       arena,
-      EMIT_FLAGS_BACKWARDS,
+      emit_flags | EMIT_BACKWARDS,
       NULL,
       &code_size));
 
+  total_size += code_size;
+
   FAIL_ON_ERROR(_yr_emit_inst(
       arena,
       RE_OPCODE_MATCH,
       NULL,
       &code_size));
 
+  total_size += code_size;
+
+  assert(total_size < RE_MAX_CODE_SIZE);
+
   return ERROR_SUCCESS;
 }
 
@@ -963,7 +1104,7 @@ int yr_re_emit_code(
 int _yr_re_alloc_storage(
     RE_THREAD_STORAGE** storage)
 {
-  #ifdef WIN32
+  #ifdef _WIN32
   *storage = TlsGetValue(thread_storage_key);
   #else
   *storage = pthread_getspecific(thread_storage_key);
@@ -979,7 +1120,7 @@ int _yr_re_alloc_storage(
     (*storage)->fiber_pool.head = NULL;
     (*storage)->fiber_pool.tail = NULL;
 
-    #ifdef WIN32
+    #ifdef _WIN32
     TlsSetValue(thread_storage_key, *storage);
     #else
     pthread_setspecific(thread_storage_key, *storage);
@@ -1222,7 +1363,7 @@ void _yr_re_fiber_kill_tail(
 //
 // _yr_re_fiber_kill_tail
 //
-// Kills all fibers from in the fiber list.
+// Kills all fibers in the fiber list.
 //
 
 void _yr_re_fiber_kill_all(
@@ -1311,7 +1452,7 @@ void _yr_re_fiber_sync(
 // Executes a regular expression
 //
 // Args:
-//   uint8_t* code                    - Pointer to regexp code be executed
+//   RE_CODE re_code                  - Regexp code be executed
 //   uint8_t* input                   - Pointer to input data
 //   size_t input_size                - Input data size
 //   int flags                        - Flags:
@@ -1319,6 +1460,7 @@ void _yr_re_fiber_sync(
 //      RE_FLAGS_BACKWARDS
 //      RE_FLAGS_EXHAUSTIVE
 //      RE_FLAGS_WIDE
+//      RE_FLAGS_NOT_AT_START
 //   RE_MATCH_CALLBACK_FUNC callback  - Callback function
 //   void* callback_args              - Callback argument
 //
@@ -1326,22 +1468,21 @@ void _yr_re_fiber_sync(
 //    Integer indicating the number of matching bytes, including 0 when
 //    matching an empty regexp. Negative values indicate:
 //      -1  No match
-//      -2  Insuficient memory
-//      -3  Fatal error
+//      -2  An error ocurred
 
 int yr_re_exec(
-    uint8_t* code,
+    RE_CODE re_code,
     uint8_t* input_data,
     size_t input_size,
     int flags,
     RE_MATCH_CALLBACK_FUNC callback,
     void* callback_args)
 {
-  uint8_t* ip;
   uint8_t* input;
   uint8_t mask;
   uint8_t value;
 
+  RE_CODE ip;
   RE_FIBER_LIST fibers;
   RE_THREAD_STORAGE* storage;
   RE_FIBER* fiber;
@@ -1351,8 +1492,21 @@ int yr_re_exec(
   int max_count;
   int match;
   int character_size;
+  int kill;
+  int action;
   int result = -1;
 
+  #define ACTION_NONE       0
+  #define ACTION_CONTINUE   1
+  #define ACTION_KILL       2
+  #define ACTION_KILL_TAIL  3
+
+  #define prolog if (count >= max_count) \
+      { \
+        action = ACTION_KILL; \
+        break; \
+      }
+
   if (_yr_re_alloc_storage(&storage) != ERROR_SUCCESS)
     return -2;
 
@@ -1361,12 +1515,16 @@ int yr_re_exec(
   else
     character_size = 1;
 
-  max_count = min(input_size, RE_SCAN_LIMIT);
   input = input_data;
+
+  if (flags & RE_FLAGS_BACKWARDS)
+    input -= character_size;
+
+  max_count = min(input_size, RE_SCAN_LIMIT);
   count = 0;
 
   fiber = _yr_re_fiber_create(&storage->fiber_pool);
-  fiber->ip = code;
+  fiber->ip = re_code;
 
   fibers.head = fiber;
   fibers.tail = fiber;
@@ -1380,62 +1538,39 @@ int yr_re_exec(
     while(fiber != NULL)
     {
       ip = fiber->ip;
-
-      if (*ip == RE_OPCODE_MATCH ||
-          *ip == RE_OPCODE_MATCH_AT_START ||
-          *ip == RE_OPCODE_MATCH_AT_END)
-      {
-        if ((*ip == RE_OPCODE_MATCH_AT_START &&
-             input_size - 1 > count - character_size) ||
-            (*ip == RE_OPCODE_MATCH_AT_END &&
-             input_size > count))
-        {
-          fiber = _yr_re_fiber_kill(&fibers, &storage->fiber_pool, fiber);
-          continue;
-        }
-
-        result = count;
-
-        if (flags & RE_FLAGS_EXHAUSTIVE)
-        {
-          if (flags & RE_FLAGS_BACKWARDS)
-            callback(input + character_size, count, flags, callback_args);
-          else
-            callback(input_data, count, flags, callback_args);
-
-          fiber = _yr_re_fiber_kill(&fibers, &storage->fiber_pool, fiber);
-        }
-        else
-        {
-          _yr_re_fiber_kill_tail(&fibers, &storage->fiber_pool, fiber);
-          fiber = NULL;
-        }
-
-        continue;
-      }
-
-      if (count >= max_count)
-      {
-        fiber = _yr_re_fiber_kill(&fibers, &storage->fiber_pool, fiber);
-        continue;
-      }
+      action = ACTION_NONE;
 
       switch(*ip)
       {
         case RE_OPCODE_ANY:
-          match = (*input != 0x0A || flags & RE_FLAGS_DOT_ALL);
+          prolog;
+          action = ACTION_NONE;
+          fiber->ip += 1;
+          break;
+
+        case RE_OPCODE_ANY_EXCEPT_NEW_LINE:
+          prolog;
+          match = (*input != 0x0A);
+          action = match ? ACTION_NONE : ACTION_KILL;
           fiber->ip += 1;
           break;
 
         case RE_OPCODE_LITERAL:
-          if (flags & RE_FLAGS_NO_CASE)
-            match = lowercase[*input] == lowercase[*(ip + 1)];
-          else
-            match = (*input == *(ip + 1));
+          prolog;
+          match = (*input == *(ip + 1));
+          action = match ? ACTION_NONE : ACTION_KILL;
+          fiber->ip += 2;
+          break;
+
+        case RE_OPCODE_LITERAL_NO_CASE:
+          prolog;
+          match = lowercase[*input] == lowercase[*(ip + 1)];
+          action = match ? ACTION_NONE : ACTION_KILL;
           fiber->ip += 2;
           break;
 
         case RE_OPCODE_MASKED_LITERAL:
+          prolog;
           value = *(int16_t*)(ip + 1) & 0xFF;
           mask = *(int16_t*)(ip + 1) >> 8;
 
@@ -1444,61 +1579,154 @@ int yr_re_exec(
           // which can't be case-insensitive.
 
           match = ((*input & mask) == value);
+          action = match ? ACTION_NONE : ACTION_KILL;
           fiber->ip += 3;
           break;
 
         case RE_OPCODE_CLASS:
-          if (flags & RE_FLAGS_NO_CASE)
-            match = CHAR_IN_CLASS(*input, ip + 1) ||
-                    CHAR_IN_CLASS(altercase[*input], ip + 1);
-          else
-            match = CHAR_IN_CLASS(*input, ip + 1);
+          prolog;
+          match = CHAR_IN_CLASS(*input, ip + 1);
+          action = match ? ACTION_NONE : ACTION_KILL;
+          fiber->ip += 33;
+          break;
+
+        case RE_OPCODE_CLASS_NO_CASE:
+          prolog;
+          match = CHAR_IN_CLASS(*input, ip + 1) ||
+                  CHAR_IN_CLASS(altercase[*input], ip + 1);
+          action = match ? ACTION_NONE : ACTION_KILL;
           fiber->ip += 33;
           break;
 
         case RE_OPCODE_WORD_CHAR:
+          prolog;
           match = (isalnum(*input) || *input == '_');
+          action = match ? ACTION_NONE : ACTION_KILL;
           fiber->ip += 1;
           break;
 
         case RE_OPCODE_NON_WORD_CHAR:
+          prolog;
           match = (!isalnum(*input) && *input != '_');
+          action = match ? ACTION_NONE : ACTION_KILL;
           fiber->ip += 1;
           break;
 
         case RE_OPCODE_SPACE:
-          match = (*input == ' ' || *input == '\t');
+          prolog;
+          switch(*input)
+          {
+            case ' ':
+            case '\t':
+            case '\r':
+            case '\n':
+            case '\v':
+            case '\f':
+              match = TRUE;
+              break;
+
+            default:
+              match = FALSE;
+          }
+          action = match ? ACTION_NONE : ACTION_KILL;
           fiber->ip += 1;
           break;
 
         case RE_OPCODE_NON_SPACE:
-          match = (*input != ' ' && *input != '\t');
+          prolog;
+          switch(*input)
+          {
+            case ' ':
+            case '\t':
+            case '\r':
+            case '\n':
+            case '\v':
+            case '\f':
+              match = FALSE;
+              break;
+
+            default:
+              match = TRUE;
+          }
+          action = match ? ACTION_NONE : ACTION_KILL;
           fiber->ip += 1;
           break;
 
         case RE_OPCODE_DIGIT:
+          prolog;
           match = isdigit(*input);
+          action = match ? ACTION_NONE : ACTION_KILL;
           fiber->ip += 1;
           break;
 
         case RE_OPCODE_NON_DIGIT:
+          prolog;
           match = !isdigit(*input);
+          action = match ? ACTION_NONE : ACTION_KILL;
           fiber->ip += 1;
           break;
 
+        case RE_OPCODE_MATCH_AT_START:
+          if (flags & RE_FLAGS_BACKWARDS)
+            kill = input_size > count;
+          else
+            kill = (flags & RE_FLAGS_NOT_AT_START) || (count != 0);
+          action = kill ? ACTION_KILL : ACTION_CONTINUE;
+          break;
+
+        case RE_OPCODE_MATCH_AT_END:
+          action = input_size > count ? ACTION_KILL : ACTION_CONTINUE;
+          break;
+
+        case RE_OPCODE_MATCH:
+          result = count;
+
+          if (flags & RE_FLAGS_EXHAUSTIVE)
+          {
+            if (callback != NULL)
+            {
+              if (flags & RE_FLAGS_BACKWARDS)
+                callback(input + character_size, count,
+                         flags, callback_args);
+              else
+                callback(input_data, count,
+                         flags, callback_args);
+            }
+
+            action = ACTION_KILL;
+          }
+          else
+          {
+            action = ACTION_KILL_TAIL;
+          }
+
+          break;
+
         default:
           assert(FALSE);
       }
 
-      if (!match)
+      switch(action)
       {
-        fiber = _yr_re_fiber_kill(&fibers, &storage->fiber_pool, fiber);
-        continue;
-      }
+        case ACTION_KILL:
+          fiber = _yr_re_fiber_kill(&fibers, &storage->fiber_pool, fiber);
+          break;
 
-      next_fiber = fiber->next;
-      _yr_re_fiber_sync(&fibers, &storage->fiber_pool, fiber);
-      fiber = next_fiber;
+        case ACTION_KILL_TAIL:
+          _yr_re_fiber_kill_tail(&fibers, &storage->fiber_pool, fiber);
+          fiber = NULL;
+          break;
+
+        case ACTION_CONTINUE:
+          fiber->ip += 1;
+          _yr_re_fiber_sync(&fibers, &storage->fiber_pool, fiber);
+          break;
+
+        default:
+          next_fiber = fiber->next;
+          _yr_re_fiber_sync(&fibers, &storage->fiber_pool, fiber);
+          fiber = next_fiber;
+      }
     }
 
     if (flags & RE_FLAGS_WIDE && count + 1 < max_count && *(input + 1) != 0)
@@ -1514,7 +1742,7 @@ int yr_re_exec(
     if (flags & RE_FLAGS_SCAN && count < max_count)
     {
       fiber = _yr_re_fiber_create(&storage->fiber_pool);
-      fiber->ip = code;
+      fiber->ip = re_code;
 
       _yr_re_fiber_append(&fibers, fiber);
       _yr_re_fiber_sync(&fibers, &storage->fiber_pool, fiber);
