@@ -107,30 +107,20 @@
 
 #include <stdint.h>
 
-#include "mem.h"
-#include "re_lexer.h"
-#include "re.h"
+#include <yara/utils.h>
+#include <yara/error.h>
+#include <yara/limits.h>
+#include <yara/mem.h>
+#include <yara/re.h>
+#include <yara/re_lexer.h>
 
-#include "config.h"
-
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
 
 #define YYERROR_VERBOSE
 
-
-#define YYDEBUG 0
-
-#if YYDEBUG
-yydebug = 1;
-#endif
-
 #define ERROR_IF(x, error) \
     if (x) \
     { \
-      RE* re = yyget_extra(yyscanner); \
-      re->error_code = error; \
+      lex_env->last_error_code = error; \
       YYABORT; \
     } \
 
@@ -162,7 +152,7 @@ yydebug = 1;
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 67 "re_grammar.y"
+#line 57 "re_grammar.y"
 {
   int integer;
   uint32_t range;
@@ -170,7 +160,7 @@ typedef union YYSTYPE
   uint8_t* class_vector;
 }
 /* Line 193 of yacc.c.  */
-#line 174 "re_grammar.c"
+#line 164 "re_grammar.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -183,7 +173,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 187 "re_grammar.c"
+#line 177 "re_grammar.c"
 
 #ifdef short
 # undef short
@@ -474,9 +464,9 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,    96,    96,   101,   104,   108,   117,   132,   136,   146,
-     153,   162,   169,   178,   188,   199,   209,   213,   219,   227,
-     231,   237,   245,   251,   257,   263,   269,   275,   281
+       0,    86,    86,    91,    94,    98,   107,   122,   126,   136,
+     143,   152,   159,   168,   178,   189,   199,   203,   209,   217,
+     221,   227,   235,   241,   247,   253,   259,   265,   271
 };
 #endif
 
@@ -711,7 +701,7 @@ do {									  \
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep, void *yyscanner, LEX_ENVIRONMENT *lex_env)
+yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep, void *yyscanner, RE_LEX_ENVIRONMENT *lex_env)
 #else
 static void
 yy_symbol_value_print (yyoutput, yytype, yyvaluep, yyscanner, lex_env)
@@ -719,7 +709,7 @@ yy_symbol_value_print (yyoutput, yytype, yyvaluep, yyscanner, lex_env)
     int yytype;
     YYSTYPE const * const yyvaluep;
     void *yyscanner;
-    LEX_ENVIRONMENT *lex_env;
+    RE_LEX_ENVIRONMENT *lex_env;
 #endif
 {
   if (!yyvaluep)
@@ -747,7 +737,7 @@ yy_symbol_value_print (yyoutput, yytype, yyvaluep, yyscanner, lex_env)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep, void *yyscanner, LEX_ENVIRONMENT *lex_env)
+yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep, void *yyscanner, RE_LEX_ENVIRONMENT *lex_env)
 #else
 static void
 yy_symbol_print (yyoutput, yytype, yyvaluep, yyscanner, lex_env)
@@ -755,7 +745,7 @@ yy_symbol_print (yyoutput, yytype, yyvaluep, yyscanner, lex_env)
     int yytype;
     YYSTYPE const * const yyvaluep;
     void *yyscanner;
-    LEX_ENVIRONMENT *lex_env;
+    RE_LEX_ENVIRONMENT *lex_env;
 #endif
 {
   if (yytype < YYNTOKENS)
@@ -803,14 +793,14 @@ do {								\
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_reduce_print (YYSTYPE *yyvsp, int yyrule, void *yyscanner, LEX_ENVIRONMENT *lex_env)
+yy_reduce_print (YYSTYPE *yyvsp, int yyrule, void *yyscanner, RE_LEX_ENVIRONMENT *lex_env)
 #else
 static void
 yy_reduce_print (yyvsp, yyrule, yyscanner, lex_env)
     YYSTYPE *yyvsp;
     int yyrule;
     void *yyscanner;
-    LEX_ENVIRONMENT *lex_env;
+    RE_LEX_ENVIRONMENT *lex_env;
 #endif
 {
   int yynrhs = yyr2[yyrule];
@@ -1083,7 +1073,7 @@ yysyntax_error (char *yyresult, int yystate, int yychar)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep, void *yyscanner, LEX_ENVIRONMENT *lex_env)
+yydestruct (const char *yymsg, int yytype, YYSTYPE *yyvaluep, void *yyscanner, RE_LEX_ENVIRONMENT *lex_env)
 #else
 static void
 yydestruct (yymsg, yytype, yyvaluep, yyscanner, lex_env)
@@ -1091,7 +1081,7 @@ yydestruct (yymsg, yytype, yyvaluep, yyscanner, lex_env)
     int yytype;
     YYSTYPE *yyvaluep;
     void *yyscanner;
-    LEX_ENVIRONMENT *lex_env;
+    RE_LEX_ENVIRONMENT *lex_env;
 #endif
 {
   YYUSE (yyvaluep);
@@ -1105,30 +1095,30 @@ yydestruct (yymsg, yytype, yyvaluep, yyscanner, lex_env)
   switch (yytype)
     {
       case 6: /* "_CLASS_" */
-#line 88 "re_grammar.y"
+#line 78 "re_grammar.y"
 	{ yr_free((yyvaluep->class_vector)); };
-#line 1111 "re_grammar.c"
+#line 1101 "re_grammar.c"
 	break;
       case 24: /* "alternative" */
-#line 89 "re_grammar.y"
+#line 79 "re_grammar.y"
+	{ yr_re_node_destroy((yyvaluep->re_node)); };
+#line 1106 "re_grammar.c"
+	break;
+      case 25: /* "concatenation" */
+#line 80 "re_grammar.y"
+	{ yr_re_node_destroy((yyvaluep->re_node)); };
+#line 1111 "re_grammar.c"
+	break;
+      case 26: /* "repeat" */
+#line 81 "re_grammar.y"
 	{ yr_re_node_destroy((yyvaluep->re_node)); };
 #line 1116 "re_grammar.c"
 	break;
-      case 25: /* "concatenation" */
-#line 90 "re_grammar.y"
+      case 27: /* "single" */
+#line 82 "re_grammar.y"
 	{ yr_re_node_destroy((yyvaluep->re_node)); };
 #line 1121 "re_grammar.c"
 	break;
-      case 26: /* "repeat" */
-#line 91 "re_grammar.y"
-	{ yr_re_node_destroy((yyvaluep->re_node)); };
-#line 1126 "re_grammar.c"
-	break;
-      case 27: /* "single" */
-#line 92 "re_grammar.y"
-	{ yr_re_node_destroy((yyvaluep->re_node)); };
-#line 1131 "re_grammar.c"
-	break;
 
       default:
 	break;
@@ -1146,7 +1136,7 @@ int yyparse ();
 #endif
 #else /* ! YYPARSE_PARAM */
 #if defined __STDC__ || defined __cplusplus
-int yyparse (void *yyscanner, LEX_ENVIRONMENT *lex_env);
+int yyparse (void *yyscanner, RE_LEX_ENVIRONMENT *lex_env);
 #else
 int yyparse ();
 #endif
@@ -1175,12 +1165,12 @@ yyparse (YYPARSE_PARAM)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 int
-yyparse (void *yyscanner, LEX_ENVIRONMENT *lex_env)
+yyparse (void *yyscanner, RE_LEX_ENVIRONMENT *lex_env)
 #else
 int
 yyparse (yyscanner, lex_env)
     void *yyscanner;
-    LEX_ENVIRONMENT *lex_env;
+    RE_LEX_ENVIRONMENT *lex_env;
 #endif
 #endif
 {
@@ -1437,7 +1427,7 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 97 "re_grammar.y"
+#line 87 "re_grammar.y"
     {
         RE* re = yyget_extra(yyscanner);
         re->root_node = (yyvsp[(1) - (1)].re_node);
@@ -1445,14 +1435,14 @@ yyreduce:
     break;
 
   case 4:
-#line 105 "re_grammar.y"
+#line 95 "re_grammar.y"
     {
                 (yyval.re_node) = (yyvsp[(1) - (1)].re_node);
               }
     break;
 
   case 5:
-#line 109 "re_grammar.y"
+#line 99 "re_grammar.y"
     {
                 (yyval.re_node) = yr_re_node_create(RE_NODE_ALT, (yyvsp[(1) - (3)].re_node), (yyvsp[(3) - (3)].re_node));
 
@@ -1464,7 +1454,7 @@ yyreduce:
     break;
 
   case 6:
-#line 118 "re_grammar.y"
+#line 108 "re_grammar.y"
     {
                 RE_NODE* node;
 
@@ -1480,14 +1470,14 @@ yyreduce:
     break;
 
   case 7:
-#line 133 "re_grammar.y"
+#line 123 "re_grammar.y"
     {
                   (yyval.re_node) = (yyvsp[(1) - (1)].re_node);
                 }
     break;
 
   case 8:
-#line 137 "re_grammar.y"
+#line 127 "re_grammar.y"
     {
                   (yyval.re_node) = yr_re_node_create(RE_NODE_CONCAT, (yyvsp[(1) - (2)].re_node), (yyvsp[(2) - (2)].re_node));
 
@@ -1498,7 +1488,7 @@ yyreduce:
     break;
 
   case 9:
-#line 147 "re_grammar.y"
+#line 137 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_STAR, (yyvsp[(1) - (2)].re_node), NULL);
 
@@ -1508,7 +1498,7 @@ yyreduce:
     break;
 
   case 10:
-#line 154 "re_grammar.y"
+#line 144 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_STAR, (yyvsp[(1) - (3)].re_node), NULL);
 
@@ -1520,7 +1510,7 @@ yyreduce:
     break;
 
   case 11:
-#line 163 "re_grammar.y"
+#line 153 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_PLUS, (yyvsp[(1) - (2)].re_node), NULL);
 
@@ -1530,7 +1520,7 @@ yyreduce:
     break;
 
   case 12:
-#line 170 "re_grammar.y"
+#line 160 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_PLUS, (yyvsp[(1) - (3)].re_node), NULL);
 
@@ -1542,7 +1532,7 @@ yyreduce:
     break;
 
   case 13:
-#line 179 "re_grammar.y"
+#line 169 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_RANGE, (yyvsp[(1) - (2)].re_node), NULL);
 
@@ -1555,7 +1545,7 @@ yyreduce:
     break;
 
   case 14:
-#line 189 "re_grammar.y"
+#line 179 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_RANGE, (yyvsp[(1) - (3)].re_node), NULL);
 
@@ -1569,7 +1559,7 @@ yyreduce:
     break;
 
   case 15:
-#line 200 "re_grammar.y"
+#line 190 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_RANGE, (yyvsp[(1) - (2)].re_node), NULL);
 
@@ -1582,14 +1572,14 @@ yyreduce:
     break;
 
   case 16:
-#line 210 "re_grammar.y"
+#line 200 "re_grammar.y"
     {
             (yyval.re_node) = (yyvsp[(1) - (1)].re_node);
          }
     break;
 
   case 17:
-#line 214 "re_grammar.y"
+#line 204 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_ANCHOR_START, NULL, NULL);
 
@@ -1598,7 +1588,7 @@ yyreduce:
     break;
 
   case 18:
-#line 220 "re_grammar.y"
+#line 210 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_ANCHOR_END, NULL, NULL);
 
@@ -1607,14 +1597,14 @@ yyreduce:
     break;
 
   case 19:
-#line 228 "re_grammar.y"
+#line 218 "re_grammar.y"
     {
             (yyval.re_node) = (yyvsp[(2) - (3)].re_node);
          }
     break;
 
   case 20:
-#line 232 "re_grammar.y"
+#line 222 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_ANY, NULL, NULL);
 
@@ -1623,7 +1613,7 @@ yyreduce:
     break;
 
   case 21:
-#line 238 "re_grammar.y"
+#line 228 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_LITERAL, NULL, NULL);
 
@@ -1634,7 +1624,7 @@ yyreduce:
     break;
 
   case 22:
-#line 246 "re_grammar.y"
+#line 236 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_WORD_CHAR, NULL, NULL);
 
@@ -1643,7 +1633,7 @@ yyreduce:
     break;
 
   case 23:
-#line 252 "re_grammar.y"
+#line 242 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_NON_WORD_CHAR, NULL, NULL);
 
@@ -1652,7 +1642,7 @@ yyreduce:
     break;
 
   case 24:
-#line 258 "re_grammar.y"
+#line 248 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_SPACE, NULL, NULL);
 
@@ -1661,7 +1651,7 @@ yyreduce:
     break;
 
   case 25:
-#line 264 "re_grammar.y"
+#line 254 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_NON_SPACE, NULL, NULL);
 
@@ -1670,7 +1660,7 @@ yyreduce:
     break;
 
   case 26:
-#line 270 "re_grammar.y"
+#line 260 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_DIGIT, NULL, NULL);
 
@@ -1679,7 +1669,7 @@ yyreduce:
     break;
 
   case 27:
-#line 276 "re_grammar.y"
+#line 266 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_NON_DIGIT, NULL, NULL);
 
@@ -1688,7 +1678,7 @@ yyreduce:
     break;
 
   case 28:
-#line 282 "re_grammar.y"
+#line 272 "re_grammar.y"
     {
             (yyval.re_node) = yr_re_node_create(RE_NODE_CLASS, NULL, NULL);
 
@@ -1700,7 +1690,7 @@ yyreduce:
 
 
 /* Line 1267 of yacc.c.  */
-#line 1704 "re_grammar.c"
+#line 1694 "re_grammar.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1914,22 +1904,6 @@ yyreturn:
 }
 
 
-#line 292 "re_grammar.y"
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+#line 282 "re_grammar.y"
 
 
