@@ -17,12 +17,12 @@
 from distutils.core import setup, Extension
 
 setup(name='yara-python',
-      version='2.1',
+      version='3.1',
       author='Victor M. Alvarez',
       author_email='plusvic@gmail.com;vmalvarez@virustotal.com',
       ext_modules=[Extension(
         name='yara',
         sources=['yara-python.c'],
         libraries=['yara'],
-        include_dirs=['../windows/include', '../libyara'],
+        include_dirs=['../windows/include', '../libyara/include'],
     )])
