@@ -17,14 +17,13 @@
 from distutils.core import setup, Extension
 
 setup(name='yara-python',
-      version='2.1',
+      version='3.1',
       author='Victor M. Alvarez',
       author_email='plusvic@gmail.com;vmalvarez@virustotal.com',
       ext_modules=[Extension(
         name='yara',
         sources=['yara-python.c'],
-        include_dirs=['../windows/include', '../libyara'],
-        define_macros=[('WIN32', None)],
+        include_dirs=['../windows/include', '../libyara/include'],
         extra_objects=[
           '../windows/libyara/Release/libyara32.lib']
         )])
